'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var apolloServerExpress = require('apollo-server-express');
var React = _interopDefault(require('react'));
var ReactDOMServer = _interopDefault(require('react-dom/server'));
var apolloClient = require('apollo-client');
var apolloLinkSchema = require('apollo-link-schema');
var graphqlTools = require('graphql-tools');
var apolloCacheInmemory = require('apollo-cache-inmemory');
var reactHooks = require('@apollo/react-hooks');
var reactSsr = require('@apollo/react-ssr');

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var LIB_TAG = "universal-react-apollo";
var REHYDRATION_STATE_DATA_KEY = "__APOLLO_STATE__";
var ROOT_CONTAINER_ID = "app-root";
var DEFAULT_STATUS_CODE = 200;

var Html = function Html(_ref) {
  var content = _ref.content,
      initialState = _ref.initialState,
      head = _ref.head,
      bodyBottomElement = _ref.bodyBottomElement,
      _ref$htmlTagAttrs = _ref.htmlTagAttrs,
      htmlTagAttrs = _ref$htmlTagAttrs === void 0 ? {} : _ref$htmlTagAttrs,
      inlineStateNonce = _ref.inlineStateNonce;
  return React.createElement("html", htmlTagAttrs, React.createElement("head", {
    dangerouslySetInnerHTML: {
      __html: head
    }
  }), React.createElement("body", null, React.createElement("div", {
    id: ROOT_CONTAINER_ID,
    dangerouslySetInnerHTML: {
      __html: content
    }
  }), React.createElement("script", {
    nonce: inlineStateNonce,
    dangerouslySetInnerHTML: {
      __html: "window.".concat(REHYDRATION_STATE_DATA_KEY, "=").concat(JSON.stringify(initialState).replace(/</g, "\\u003c"))
    }
  }), bodyBottomElement));
};

/**
 * Return a function which is responsible for fetching all required states via graphql operations and render application with them in server side
 * @param {Object} options Server side rendering options
 * @param {String} options.typeDefs GraphQL schema language string, array of GraphQL schema language strings or a function that return an array of GraphQL schema strings
 * @param {Object} options.resolvers GQL resolvers object - optional
 * @param {Object} options.schemaDirectives schema directive definition object
 * @return {Function} server render function
 */

function createServerRender(_ref) {
  var typeDefs = _ref.typeDefs,
      resolvers = _ref.resolvers,
      schemaDirectives = _ref.schemaDirectives;
  // create Apollo client
  var schema = graphqlTools.makeExecutableSchema({
    typeDefs: typeDefs,
    resolvers: resolvers,
    schemaDirectives: schemaDirectives
  });
  /**
   * Server render function
   * @param {Object} options
   * @param {Object} options.dataSources GQL data sources object
   * @param {Object} options.context context object which will be shared across all resolvers
   * @param {Object} options.htmlTagAttrs dom attributes of html tag
   * @param {Function} options.appElement: A function called with the current request that returns a React Element which will be placed in the <body>
   * @param {Function} options.headElement A function called with the current request that returns a React Element which will be placed in the <head>
   * @param {Function} options.bodyBottomElement A function called with the current request that returns a React Element which will be placed in the bottom of the appElement
   * @param {Object} options.req Express request object
   * @param {Object} options.res Express response object
   * @param {Object} options.cache GQL data source cache config object - optional
   */

  return (
    /*#__PURE__*/
    function () {
      var _ref3 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(_ref2) {
        var dataSources, cache, context, htmlTagAttrs, appElement, headElement, bodyBottomElement, req, res, _i, _Object$values, dataSource, apolloClientOptions, client, ssrClient, headWithApollo, appWithApollo, app, head, initialState, html;

        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                dataSources = _ref2.dataSources, cache = _ref2.cache, context = _ref2.context, htmlTagAttrs = _ref2.htmlTagAttrs, appElement = _ref2.appElement, headElement = _ref2.headElement, bodyBottomElement = _ref2.bodyBottomElement, req = _ref2.req, res = _ref2.res;

                // initialize the data source, required for Apollo RESTDataSource
                for (_i = 0, _Object$values = Object.values(dataSources); _i < _Object$values.length; _i++) {
                  dataSource = _Object$values[_i];

                  if (typeof dataSource.initialize === "function") {
                    dataSource.initialize({
                      context: context,
                      cache: cache
                    });
                  }
                } // prepare apollo client


                apolloClientOptions = {
                  ssrMode: true,
                  link: new apolloLinkSchema.SchemaLink({
                    schema: schema,
                    context: _objectSpread2({}, context, {
                      dataSources: dataSources
                    })
                  }),
                  cache: new apolloCacheInmemory.InMemoryCache(),
                  defaultOptions: {
                    watchQuery: {
                      errorPolicy: "all"
                    },
                    query: {
                      errorPolicy: "all"
                    },
                    mutate: {
                      errorPolicy: "all"
                    }
                  }
                };
                client = new apolloClient.ApolloClient(apolloClientOptions); // client for ssr only (no need to do data extraction)

                ssrClient = new apolloClient.ApolloClient(apolloClientOptions); // wrapping head component with Apollo Provider

                headWithApollo = React.createElement(reactHooks.ApolloProvider, {
                  client: ssrClient
                }, typeof headElement === "function" ? headElement({
                  req: req
                }) : null); // wrapping main component with Apollo Provider

                appWithApollo = React.createElement(reactHooks.ApolloProvider, {
                  client: client
                }, typeof appElement === "function" && appElement({
                  req: req,
                  res: res
                }));
                _context.next = 9;
                return reactSsr.renderToStringWithData(appWithApollo);

              case 9:
                app = _context.sent;
                _context.next = 12;
                return reactSsr.getDataFromTree(headWithApollo);

              case 12:
                head = _context.sent;
                initialState = client.extract();
                html = React.createElement(Html, {
                  content: app,
                  initialState: initialState,
                  htmlTagAttrs: htmlTagAttrs,
                  head: head,
                  bodyBottomElement: typeof bodyBottomElement === "function" ? bodyBottomElement({
                    req: req
                  }) : null,
                  inlineStateNonce: req.nonce
                });
                return _context.abrupt("return", "<!doctype html>\n".concat(ReactDOMServer.renderToStaticMarkup(html)));

              case 16:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x) {
        return _ref3.apply(this, arguments);
      };
    }()
  );
}

var initServer = function initServer(app, routes, apolloServerOptions, production) {
  var cors = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
  // mount apollo-server-express middleware
  var apolloServer = new apolloServerExpress.ApolloServer(_objectSpread2({}, apolloServerOptions, {
    uploads: false,
    playground: !production
  }));
  apolloServer.applyMiddleware({
    app: app,
    cors: cors
  });
  var typeDefs = apolloServerOptions.typeDefs,
      resolvers = apolloServerOptions.resolvers,
      dataSources = apolloServerOptions.dataSources,
      context = apolloServerOptions.context,
      schemaDirectives = apolloServerOptions.schemaDirectives; // create server render

  var serverRender = createServerRender({
    typeDefs: typeDefs,
    resolvers: resolvers,
    schemaDirectives: schemaDirectives
  }); // mount application routing

  if (Array.isArray(routes)) {
    routes.forEach(function (_ref) {
      var path = _ref.path,
          _ref$method = _ref.method,
          method = _ref$method === void 0 ? "get" : _ref$method,
          htmlTagAttrs = _ref.htmlTagAttrs,
          appElement = _ref.appElement,
          headElement = _ref.headElement,
          bodyBottomElement = _ref.bodyBottomElement,
          _ref$middlewareChain = _ref.middlewareChain,
          middlewareChain = _ref$middlewareChain === void 0 ? [] : _ref$middlewareChain,
          _ref$responseStatusCo = _ref.responseStatusCode,
          responseStatusCode = _ref$responseStatusCo === void 0 ? DEFAULT_STATUS_CODE : _ref$responseStatusCo;
      app[method.toLowerCase()](path, middlewareChain, function (req, res, next) {
        serverRender({
          htmlTagAttrs: htmlTagAttrs,
          appElement: appElement,
          headElement: headElement,
          bodyBottomElement: bodyBottomElement,
          req: req,
          res: res,
          cache: apolloServer.requestOptions.cache,
          dataSources: typeof dataSources === "function" ? dataSources() : null,
          context: typeof context === "function" ? context({
            req: req,
            connection: {},
            res: res
          }) : context
        }).then(function (html) {
          if (res.statusCode === DEFAULT_STATUS_CODE && res.statusCode !== responseStatusCode) {
            res.status(responseStatusCode);
          }

          res.send(html);
        })["catch"](function (error) {
          next(error);
        });
      });
    });
  } else {
    console.error("[".concat(LIB_TAG, "] Required argument - routes is not present or not an array"));
  }
};

exports.initServer = initServer;
//# sourceMappingURL=universal-react-apollo.cjs.js.map
